<?php

namespace Drupal\pdf_using_mpdf\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\NodeInterface;
use Drupal\pdf_using_mpdf\Conversion\ConvertToPdf;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Access\AccessResult;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;

/**
 * Class GeneratePdf.
 */
class GeneratePdf extends ControllerBase implements ContainerInjectionInterface {

  /**
   * Convert to PDF service.
   *
   * @var \Drupal\pdf_using_mpdf\Conversion\ConvertToPdf
   */
  protected $convert;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('pdf_using_mpdf.conversion'),
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('config.factory')
    );
  }

  /**
   * GeneratePdf constructor.
   *
   * @param \Drupal\pdf_using_mpdf\Conversion\ConvertToPdf $convert
   *   Convert to PDF service.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   The renderer service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory service.
   */
  public function __construct(
    ConvertToPdf $convert,
    EntityTypeManager $entityTypeManager,
    Renderer $renderer,
    ConfigFactoryInterface $configFactory
  ) {
    $this->convert = $convert;
    $this->entityTypeManager = $entityTypeManager;
    $this->renderer = $renderer;
    $this->configFactory = $configFactory;
  }

  /**
   * Checks if current user has access to view PDF.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Current user account.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   True if user has access, false otherwise.
   */
  public function checkAccess(AccountInterface $account) {
    return AccessResult::allowedIf($account->hasPermission('generate pdf using mpdf') && $this->checkContentTypeEnabled());
  }

  /**
   * Checks if current node is enabled for viewing in PDF.
   *
   * @return bool
   *   True if content type enabled, false otherwise.
   */
  protected function checkContentTypeEnabled() {
    $node = \Drupal::routeMatch()->getParameter('node');
    if ($node instanceof NodeInterface) {
      $config = $this->configFactory
        ->getEditable('pdf_using_mpdf.settings')
        ->get('pdf_using_mpdf');
      if (in_array($node->getType(), $config['enabled_content_types'])) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Generate a PDF file of a node.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node object.
   *
   * @return array
   *   Response headers.
   *
   * @throws \Exception
   */
  public function generate(NodeInterface $node) {
    $build = $this->entityTypeManager->getViewBuilder('node')->view($node, 'pdf_using_mpdf');
    $fileContent = $this->convert->convertToString($this->renderer->render($build));
    $response = new Response($fileContent);
    $disposition = $response->headers->makeDisposition(
      ResponseHeaderBag::DISPOSITION_ATTACHMENT, $this->convert->getFilename()
    );
    $response->headers->set('Content-Disposition', $disposition);
    $response->headers->set('Content-type', 'application/pdf');

    return $response;
  }

}
